@@ -44,7 +44,7 @@
 ;;;
 ;;; We use named pipes for the IPC to minimise implementation-specific code.
 
-(defparameter *fork-control* nil)
+(defvar *fork-control* nil)
 
 (defmacro with-fork-request (prerequest request (out err exit) &body forms)
   (with-gensyms (input output)
@@ -74,6 +74,13 @@
 ;;; carry over *HOST*, *CONNECTION* and *CONSFIGURATOR-DEBUG-LEVEL*, and in
 ;;; the latter case we do not carry over any of these by default.
 
+(defun wrap-grandchild-request (&rest forms)
+  ``(let ((*host* ,*host*)
+          (*connection* ,*connection*)
+          (*no-data-sources* t)
+          (*consfigurator-debug-level* ,*consfigurator-debug-level*))
+      ,,@forms))
+
 (defmacro eval-in-grandchild (prerequest request (out err exit) &body forms)
   "Evaluate PREREQUEST and REQUEST, both readably printable Lisp forms, in a
 grandchild process.  PREREQUEST and REQUEST must be evaluable using only
@@ -85,23 +92,86 @@ FORMS.
 PREREQUEST will be evaluated before the grandchild calls fork(2) to establish
 its own infrastructure for subsequent uses of this macro, and REQUEST after.
 Thus, PREREQUEST must not start up any threads."
-  (flet ((wrap (&rest forms)
-           ``(let ((*host* ,*host*)
-                   (*connection* ,*connection*)
-                   (*no-data-sources* t)
-                   (*consfigurator-debug-level* ,*consfigurator-debug-level*))
-               ,,@forms)))
-    `(with-fork-request
-         ,(wrap '`(posix-login-environment
-                   ,(get-connattr :remote-user) ,(get-connattr :remote-home))
-                prerequest)
-         ,(wrap request) (,out ,err ,exit)
-       ,@forms)))
+  `(with-fork-request
+       ,(wrap-grandchild-request
+         '`(posix-login-environment
+            ,(get-connattr :remote-uid)
+            ,(get-connattr :remote-user)
+            ,(get-connattr :remote-home))
+         prerequest)
+       ,(wrap-grandchild-request request) (,out ,err ,exit)
+     ,@forms))
 
-(defun dump-consfigurator (filename form)
-  (nix:umask #o077)
-  (uiop:register-image-restore-hook (lambda () (eval form)) nil)
-  (uiop:dump-image filename :executable t))
+#+sbcl (defvar *sbcl-core-cksum* (local-cksum sb-ext:*core-pathname*))
+#+sbcl (defvar *sbcl-runtime-cksum* (local-cksum sb-ext:*runtime-pathname*))
+
+(defvar *us*
+  #+sbcl sb-ext:*core-pathname*
+  #+(and linux (not sbcl)) (resolve-symlinks "/proc/self/exe"))
+
+(define-simple-error wrong-execution-context-for-image-dump (aborted-change))
+
+(defun %dump-consfigurator-in-grandchild (filename pre-dump form)
+  ;; Check that the image is likely to be reinvokable.
+  (loop for library in (list-foreign-libraries)
+        for path = (foreign-library-pathname library)
+        ;; If it's not absolute then it's something like "libacl.so" and we
+        ;; don't know whether that'll be findable at reinvocation time.
+        when (and (absolute-pathname-p path)
+                  (not (ignore-errors (nix:access path nix:R-OK))))
+          do (wrong-execution-context-for-image-dump
+              "Executable image will not be reinvokable as ~S unreadable."
+              path))
+  ;; Check that we can dump.
+  #+sbcl
+  (unless (and (not (pathname-equal filename *us*))
+               (file-exists-p sb-ext:*core-pathname*)
+               (file-exists-p sb-ext:*runtime-pathname*)
+               ;; As SBCL does not expose the build-id, use two checksums.
+               (= *sbcl-core-cksum*
+                  (local-cksum sb-ext:*core-pathname*))
+               (= *sbcl-runtime-cksum*
+                  (local-cksum sb-ext:*runtime-pathname*)))
+    (wrong-execution-context-for-image-dump
+     "Couldn't dump executable image because same SBCL build unavailable."))
+  ;; Perform the image dump.
+  (with-fork-request nil
+      `(progn ,pre-dump
+              (nix:umask #o077)
+              (uiop:register-image-restore-hook (lambda () (eval ',form)) nil)
+              (uiop:dump-image ,filename :executable t))
+      (out err exit)
+    (declare (ignore out))
+    (unless (zerop exit)
+      (failed-change "~&Failed to dump image; stderr was ~%~%~A" err))))
+
+(defmacro eval-in-reinvoked (prerequest request (out err exit) &body forms)
+  "In a grandchild process, evaluate PREREQUEST, dump an executable image, and
+immediately reinvoke that image to evaluate REQUEST.  PREREQUEST and REQUEST
+must be evaluable using only definitions established statically by your
+consfig, or in one of the ASDF systems upon which your consfig depends.  Then
+bind OUT, ERR and EXIT to the stdout, stderr and exit code of that process,
+respectively, and evaluate FORMS."
+  (with-gensyms (tempdir)
+    ;; Create a temporary directory which will be readable only by the present
+    ;; user, because of how SBCL overrides the umask when dumping an image.
+    ;; Don't want to use ~/.cache/consfigurator/images because want to write
+    ;; to a tmpfs/ramdisk if possible.
+    `(with-local-temporary-directory (,tempdir)
+       (let ((file (merge-pathnames "image" ,tempdir)))
+         (%dump-consfigurator-in-grandchild
+          file ,(wrap-grandchild-request prerequest)
+          ;; Try to ensure that the new fork control child does not end up
+          ;; with the actual request in its memory.
+          '(with-backtrace-and-exit-code
+            (with-fork-control (eval (with-standard-io-syntax (read))))))
+         (nix:chmod file #o700)         ; ensure it's executable
+         (multiple-value-bind (,out ,err ,exit)
+             (run :may-fail :input (with-standard-io-syntax
+                                     (write-to-string
+                                      ,(wrap-grandchild-request request)))
+                  file)
+           ,@forms)))))
 
 (defun dump-consfigurator-in-grandchild
     (filename &optional (form `(let ((*no-data-sources* t)
@@ -124,10 +194,7 @@ The process which performs the dump will have its umask set to #o077, but
 implementation-specific image dumping code might undo this (SBCL, for example,
 changes the mode of the file to #o755).  You might want to ensure that the
 directory containing FILENAME is locked down."
-  (with-fork-request nil `(dump-consfigurator ,filename ',form) (out err exit)
-    (declare (ignore out))
-    (unless (zerop exit)
-      (failed-change "~&Failed to dump image; stderr was ~%~%~A" err))))
+  (%dump-consfigurator-in-grandchild filename nil form))
 
 (defprop image-dumped :lisp (&optional filename form (always form))
   "Dump an executable image to FILENAME which will evaluate FORM, which must be
@@ -137,7 +204,39 @@ one of the ASDF systems upon which your consfig depends.
 If FILENAME is nil then use ~/.cache/consfigurator/images/latest, and if FORM
 is nil then use one which will execute the current deployment.  Unless ALWAYS,
 skip dumping an executable image when we can detect that the deployment is
-already running from FILENAME."
+already running from FILENAME.
+
+When the Lisp implementation is SBCL, applying this property will fail unless
+the same build of SBCL that's running is accessible via the filesystem.  A
+common situation in which this problem arises is when Consfigurator has forked
+into a chroot: unless the chroot has exactly the same operating system release
+as the parent host and SBCL has already been installed within the chroot, the
+requisite build of SBCL won't be accessible via the filesystem.
+
+Similarly, the dumped image will not be reinvokable if any of the shared
+libraries currently open are not accessible via the filesystem, which will
+usually be the case after forking into a chroot.  See \"Dumping and reinvoking
+Lisp\" in the \"Pitfalls and limitations\" section of the Consfigurator
+manual.
+
+In these situations you might like to quietly skip dumping an image.  For
+example, if you're preparing a disk image, successfully dumping an image is
+probably not needed for the image to be bootable.  After installing the disk
+image and starting up the target machine, you can connect to it directly and
+DEPLOY/HOSTDEPLOY all its properties again, at which point the image dump can
+succeed.
+
+To support this, when the current connection is a FORK-CONNECTION, and either
+the Lisp implementation is SBCL and the same build of SBCL is not found to be
+accessible via the filesytem, or some of the shared libraries currently open
+are not accessible at their original paths, this property signals
+WRONG-EXECUTION-CONTEXT-FOR-IMAGE-DUMP.  You can then quietly skip over this
+property by applying it like this:
+
+    (eseqprops-until 'wrong-execution-context-for-image-dump
+     (image-dumped)
+     ;; ... properties depending on a successful image dump ...
+     )"
   (:desc (if form
              (format nil "Dumped image to evaluate ~S" form)
              "Dumped image to execute current deployment"))
@@ -145,12 +244,23 @@ already running from FILENAME."
    (let ((file (or filename
                    (ensure-directories-exist
                     (merge-pathnames "consfigurator/images/latest"
-                                     (get-connattr :XDG-CACHE-HOME))))))
-     (unless (and (not always)
-                  (eql :linux (uiop:operating-system))
-                  (pathname-equal file (resolve-symlinks "/proc/self/exe")))
-       (unless filename
-         (nix:chmod (unix-namestring (pathname-directory-pathname file)) #o700))
+                                     (get-connattr :XDG_CACHE_HOME))))))
+     (when (and (not always) (pathname-equal file *us*))
+       (return-from image-dumped :no-change))
+     (unless filename
+       (nix:chmod (unix-namestring (pathname-directory-pathname file)) #o700))
+     (handler-bind
+         ((wrong-execution-context-for-image-dump
+            (lambda (error)
+              (unless (subtypep
+                       (class-of *connection*)
+                       'consfigurator.connection.fork:fork-connection)
+                ;; If we're *not* in one of the the cases described in the
+                ;; docstring, just signal a regular ABORTED-CHANGE, because
+                ;; this situation probably indicates a consfig problem which
+                ;; should not just be quietly skipped over.
+                (apply #'aborted-change (simple-condition-format-control error)
+                       (simple-condition-format-arguments error))))))
        (if form
            (dump-consfigurator-in-grandchild file form)
            (dump-consfigurator-in-grandchild file))))
@@ -228,7 +338,8 @@ already running from FILENAME."
 ;; a remote Lisp image and every time we establish a further FORK-CONNECTION,
 ;; however, though perhaps we could arrange to avoid dumping an executable if
 ;; we know that we aren't going to need to reinvoke it, by having connection
-;; types push hostattrs.
+;; types push hostattrs.  We would also need to resolve issues detailed under
+;; "Dumping and reinvoking Lisp" in pitfalls.rst.
 ;;
 ;; If we took this approach, then we'd have implementation-specific dumping
 ;; code, but the code to reinvoke the dumped images would be fully portable --
@@ -264,20 +375,26 @@ already running from FILENAME."
                 (eval-input)
                 (with-fork-control (eval-input)))))))
     (unwind-protect
-         (with-open-file (out out :element-type 'character)
-           (with-open-file (err err :element-type 'character)
-             (let ((status (nth-value 1 (nix:waitpid child))))
-               (unless (nix:WIFEXITED status)
-                 (failed-change
-                  "~&Grandchild process did not exit normally, status #x~(~4,'0X~)."
-                  status))
-               (with-open-file (output output :direction :output
-                                              :if-exists :append
-                                              :element-type 'character)
-                 (write-to-mkfifo (list (slurp-stream-string out)
-                                        (slurp-stream-string err)
-                                        (nix:WEXITSTATUS status))
-                                  output)))))
+         (let ((out-reader
+                 (bt:make-thread
+                  (lambda ()
+                    (read-file-string out :element-type 'character))))
+               (err-reader
+                 (bt:make-thread
+                  (lambda ()
+                    (read-file-string err :element-type 'character))))
+               (status (nth-value 1 (nix:waitpid child))))
+           (unless (nix:WIFEXITED status)
+             (failed-change
+              "~&Grandchild process did not exit normally, status #x~(~4,'0X~)."
+              status))
+           (with-open-file (output output :direction :output
+                                          :if-exists :append
+                                          :element-type 'character)
+             (write-to-mkfifo (list (bt:join-thread out-reader)
+                                    (bt:join-thread err-reader)
+                                    (nix:WEXITSTATUS status))
+                              output)))
       (delete-file out) (delete-file err))))
 
 (defclass asdf-requirements ()
@@ -448,7 +565,7 @@ Preprocessing must occur in the root Lisp."))
                            *error-output*
                            "~&Failed to compile and/or load:~%~A~&~%Compile and/or load output:~%~%~A"
                            c string)
-                          (uiop:quit 3)))
+                          (uiop:quit 1)))
                       (when (>= *consfigurator-debug-level* 3)
                         (format t "~&~A" string))))
                  ;; Continue the deployment.  The READ indirection is to try
