@@ -310,6 +310,12 @@ function cell, instead of constructing a propapp, within DEFPROPSPEC."))
                          ;; Properties with :HOSTATTRS subroutines which set
                          ;; new hostattrs should not be used programmatically
                          ;; in this way, so issue a warning.
+                         ;;
+                         ;; TODO If this property is defined using
+                         ;; DEFPROPLIST/DEFPROPSPEC and has no user-supplied
+                         ;; :HOSTATTRS subroutine of its own, but one of the
+                         ;; properties in the returned propspec does, then we
+                         ;; won't issue the warning, but we should.
                          ,@(and programmatic-warning
                                 (getf ,slotsv :hostattrs)
                                 `((warn 'programmatic-apply-hostattrs
@@ -331,7 +337,11 @@ subroutine, muffle warnings about calling a property with a :HOSTATTRS
 subroutine programmatically.  Use this only when you know that the :HOSTATTRS
 subroutine does not push any new hostattrs."
   (unless
-      (and (listp form) (or (not (fboundp (car form))) (isprop (car form))))
+      (and (listp form) (or (not (fboundp (car form)))
+                            (isprop (car form))
+                            (and (listp (cadr form))
+                                 (eql 'function (caadr form))
+                                 (isprop (cadadr form)))))
     (simple-program-error "~A is not a programmatic call to a property." form))
   `(handler-bind ((programmatic-apply-hostattrs #'muffle-warning))
      ,form))
@@ -473,7 +483,7 @@ other than constant values and propapps to property combinators."
 
 ;;;; hostattrs in property subroutines
 
-(define-simple-error inapplicable-property
+(define-simple-error inapplicable-property ()
   "Signal, in a :HOSTATTRS subroutine, that the host's hostattrs indicate that
 this property cannot be applied to this host.  E.g. the property will try to
 install an apt package but the host is FreeBSD.")
@@ -553,10 +563,18 @@ Called by property subroutines."
 
 ;;;; :APPLY subroutines
 
-(define-simple-error failed-change
+(define-simple-error failed-change ()
   "Signal problems with the connection and errors while actually attempting to
 apply or unapply properties.")
 
+(define-simple-error aborted-change (failed-change)
+  "Like FAILED-CHANGE, except the attempt to apply or unapply the property has
+failed before any changes have been made to the system.  Signalled when a
+property is able to determine that it cannot be applied/unapplied by examining
+the actual state of the host but without making any changes.
+
+Not to be confused with INAPPLICABLE-PROPERTY.")
+
 (defun maybe-writefile-string (path content &key (mode nil mode-supplied-p))
   "Wrapper around WRITEFILE which returns :NO-CHANGE and avoids writing PATH if
 PATH already has the specified CONTENT and MODE."
@@ -586,6 +604,12 @@ PATH already has the specified CONTENT and MODE."
 (defun cksum (file)
   (parse-integer (car (split-string (run "cksum" file)))))
 
+(defun local-cksum (file)
+  (parse-integer
+   (car
+    (split-string
+     (run-program `("cksum" ,(unix-namestring file)) :output :string)))))
+
 ;; this is a safe parse of ls(1) output given its POSIX specification
 (defun ls-cksum (file)
   (when-let* ((ls (ignore-errors
