@@ -98,7 +98,7 @@ subcommand of virsh(1) to convert the running domain into a transient domain."
 (I.e., if HOST is a string, ensure the domain named HOST is started; if HOST
 is a HOST value, start the libvirt domain whose name is HOST's hostname.)"
   (:desc #?"libvirt domain ${(get-hostname host)} started")
-  (:check (host-domain-started-p host))
+  (:check (or (service:no-services-p) (host-domain-started-p host)))
   (:apply (mrun "virsh" "start" (get-hostname host))))
 
 (defprop destroyed :posix (host)
@@ -217,7 +217,12 @@ Sample usage:
       (file:contains-conf-tab \"/etc/hosts\"
                               \"192.168.122.31\" \"subbox.laptop.example.com\")
       (libvirt:kvm-boots-chroot-for. (:vcpus 2 :memory 2048)
-          subbox.laptop.example.com)
+          subbox.laptop.example.com
+        ;; Apply TIMEZONE:CONFIGURED-FROM-PARENT here rather than in the
+        ;; DEFHOST above because it can be applied only during a subdeployment
+        ;; of laptop.example.com -- when we deploy subbox directly, there is
+        ;; no parent host.
+        (timezone:configured-from-parent))
       (libvirt:when-started subbox.laptop.example.com
         ;; Here we elide most of the configuration necessary to ensure that
         ;; root can SSH into the VM.  You can deploy a passwordless SSH key,
